/* $Id: ggigltest.c,v 1.1 2000/12/10 11:31:36 marcus Exp $
******************************************************************************

   Demo of the GGIGL extension.

   OpenGL code:		Sam Lantinga		[slouken@devolution.com]
   LibGGIGL glue:	Marcus Sundberg		[marcus@ggi-project.org]

   This software is placed in the public domain and can be used freely
   for any purpose. It comes without any kind of warranty, either
   expressed or implied, including, but not limited to the implied
   warranties of merchantability or fitness for a particular purpose.
   Use it at your own risk. the author is not responsible for any damage
   or consequences raised by use or inability to use this program.

******************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>

#include <GL/gl.h>
#include <ggi/gg.h>
#include <ggi/ggi.h>
/* Real applications should use #include <ggi/ggigl.h> instead! */
#include "ggigl.h"


static void
do_opengl(ggi_visual_t vis)
{
	double color = 0.0;
	float cube[8][3]= {{ 0.5,  0.5, -0.5}, 
			   { 0.5, -0.5, -0.5},
			   {-0.5, -0.5, -0.5},
			   {-0.5,  0.5, -0.5},
			   {-0.5,  0.5,  0.5},
			   { 0.5,  0.5,  0.5},
			   { 0.5, -0.5,  0.5},
			   {-0.5, -0.5,  0.5}};
	ggi_mode mode;
	int w, h;

	ggiGetMode(vis, &mode);

	w = mode.visible.x;
	h = mode.visible.y;

	glViewport( 0, 0, w, h );
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(-2.0, 2.0, -2.0, 2.0, -20.0, 20.0);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glEnable(GL_DEPTH_TEST);

	glDepthFunc(GL_LESS);

	/* Loop until done. */
	while (1) {
		GLenum gl_error;

		/* Do our drawing, too. */
		glClearColor(0.0, 0.0, 0.0, 1.0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		glBegin(GL_QUADS);
			glColor3f(1.0, 0.0, 0.0);
			glVertex3fv(cube[0]);
			glVertex3fv(cube[1]);
			glVertex3fv(cube[2]);
			glVertex3fv(cube[3]);
			
			glColor3f(0.0, 1.0, 0.0);
			glVertex3fv(cube[3]);
			glVertex3fv(cube[4]);
			glVertex3fv(cube[7]);
			glVertex3fv(cube[2]);
			
			glColor3f(0.0, 0.0, 1.0);
			glVertex3fv(cube[0]);
			glVertex3fv(cube[5]);
			glVertex3fv(cube[6]);
			glVertex3fv(cube[1]);
			
			glColor3f(0.0, 1.0, 1.0);
			glVertex3fv(cube[5]);
			glVertex3fv(cube[4]);
			glVertex3fv(cube[7]);
			glVertex3fv(cube[6]);

			glColor3f(1.0, 1.0, 0.0);
			glVertex3fv(cube[5]);
			glVertex3fv(cube[0]);
			glVertex3fv(cube[3]);
			glVertex3fv(cube[4]);

			glColor3f(1.0, 0.0, 1.0);
			glVertex3fv(cube[6]);
			glVertex3fv(cube[1]);
			glVertex3fv(cube[2]);
			glVertex3fv(cube[7]);
		glEnd();
		
		glMatrixMode(GL_MODELVIEW);
		glRotatef(5.0, 1.0, 1.0, 1.0);

		ggiglSwapBuffers(vis);

		color += 0.01;

		if (color >= 1.0) {
			color = 0.0;
		}

		/* Check for error conditions. */
		gl_error = glGetError();

		if (gl_error != GL_NO_ERROR) {
			fprintf(stderr, "testgl: OpenGL error: %d\n",
				gl_error);
		}

		ggUSleep(10);

		if (ggiKbhit(vis)) break;
	}
}


int
main(void)
{
	ggi_visual_t vis;

	/* Initialize LibGGI */
	if (ggiInit() != 0) {
		fprintf(stderr, "Unable to initialize LibGGI\n");
		exit(1);
	}

	/* Initialize GGIGL extension */
	if (ggiglInit() != 0) {
		ggiPanic("Unable to initialize LibGGIGL extension\n");
	}

	/* Open the default visual */
	if ((vis = ggiOpen(NULL)) == NULL) {
		ggiPanic("Unable to open default visual\n");
	}

	/* Turn on asynchronous mode (which should always be used) */
	ggiSetFlags(vis, GGIFLAG_ASYNC);

	/* Set the default mode */
	if (ggiSetSimpleMode(vis, GGI_AUTO, GGI_AUTO, GGI_AUTO, GT_AUTO) < 0) {
		ggiPanic("Unable to set default mode\n");
	}

	/* Attach the GGIGL extension to the visual */
	if (ggiglAttach(vis, NULL) < 0) {
		ggiPanic("Unable to attach GGIGL extension to visual\n");
	}

	/* Run OpenGL demo */
	do_opengl(vis);

	/* Close visual */
	ggiClose(vis);

	/* Deinitialize GGIGL extension */
	ggiglExit();

	/* Deinitialize LibGGI */
	ggiExit();

	return 0;
}
